@@ -34,8 +34,6 @@
  * - why do we collect a list of binary chunks and not write immediately
  *   - because we need to process subchunks, before we can set e.g. nr_of_items
  *     in parent chunk
- * - need more robustness
- *   - don't parse beyond mem-block size
  */
 
 #ifdef HAVE_CONFIG_H
@@ -72,21 +70,46 @@
 
 #define GST_CAT_DEFAULT GST_CAT_REGISTRY
 
-/* macros */
+/* count string length, but return -1 if we hit the eof */
+static gint
+_strnlen (const gchar * str, gint maxlen)
+{
+  gint len = 0;
 
-#define unpack_element(_inptr, _outptr, _element)  G_STMT_START{ \
-  _outptr = (_element *) _inptr; \
-  _inptr += sizeof (_element); \
+  if (G_UNLIKELY (len == maxlen))
+    return -1;
+
+  while (*str++ != '\0') {
+    len++;
+    if (G_UNLIKELY (len == maxlen))
+      return -1;
+  }
+  return len;
+}
+
+/* reading macros */
+
+#define unpack_element(inptr, outptr, element, endptr, error_label) G_STMT_START{ \
+  if (inptr + sizeof(element) >= endptr) \
+    goto error_label; \
+  outptr = (element *) inptr; \
+  inptr += sizeof (element); \
 }G_STMT_END
 
-#define unpack_const_string(_inptr, _outptr) G_STMT_START{\
-  _outptr = g_intern_string ((const gchar *)_inptr); \
-  _inptr += strlen(_outptr) + 1; \
+#define unpack_const_string(inptr, outptr, endptr, error_label) G_STMT_START{\
+  gint _len = _strnlen (inptr, (endptr-inptr)); \
+  if (_len == -1) \
+    goto error_label; \
+  outptr = g_intern_string ((const gchar *)inptr); \
+  inptr += _len + 1; \
 }G_STMT_END
 
-#define unpack_string(_inptr, _outptr)  G_STMT_START{\
-  _outptr = g_strdup ((gchar *)_inptr); \
-  _inptr += strlen(_outptr) + 1; \
+#define unpack_string(inptr, outptr, endptr, error_label)  G_STMT_START{\
+  gint _len = _strnlen (inptr, (endptr-inptr)); \
+  if (_len == -1) \
+    goto error_label; \
+  outptr = g_memdup ((gconstpointer)inptr, _len + 1); \
+  inptr += _len + 1; \
 }G_STMT_END
 
 #define ALIGNMENT            (sizeof (void *))
@@ -226,6 +249,10 @@ static gboolean
 gst_registry_binary_cache_finish (GstRegistry * registry,
     BinaryRegistryCache * cache, gboolean success)
 {
+  /* only fsync if we're actually going to use and rename the file below */
+  if (success && fsync (registry->cache_file) < 0)
+    goto fsync_failed;
+
   if (close (registry->cache_file) < 0)
     goto close_failed;
 
@@ -240,6 +267,7 @@ gst_registry_binary_cache_finish (GstRegistry * registry,
   GST_INFO ("Wrote binary registry cache");
   return TRUE;
 
+/* ERRORS */
 fail_after_close:
   {
     g_unlink (cache->tmp_location);
@@ -247,6 +275,11 @@ fail_after_close:
     g_free (cache);
     return FALSE;
   }
+fsync_failed:
+  {
+    GST_ERROR ("fsync() failed: %s", g_strerror (errno));
+    goto fail_after_close;
+  }
 close_failed:
   {
     GST_ERROR ("close() failed: %s", g_strerror (errno));
@@ -404,7 +437,7 @@ gst_registry_binary_save_pad_template (GList ** list,
   GstBinaryPadTemplate *pt;
   GstBinaryChunk *chk;
 
-  pt = g_malloc (sizeof (GstBinaryPadTemplate));
+  pt = g_malloc0 (sizeof (GstBinaryPadTemplate));
   chk = gst_registry_binary_make_data (pt, sizeof (GstBinaryPadTemplate));
 
   pt->presence = template->presence;
@@ -445,7 +478,7 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
     GstBinaryElementFactory *ef;
     GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
 
-    ef = g_malloc (sizeof (GstBinaryElementFactory));
+    ef = g_malloc0 (sizeof (GstBinaryElementFactory));
     chk = gst_registry_binary_make_data (ef, sizeof (GstBinaryElementFactory));
     ef->npadtemplates = ef->ninterfaces = ef->nuriprotocols = 0;
     pf = (GstBinaryPluginFeature *) ef;
@@ -499,13 +532,8 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
   } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
     GstBinaryTypeFindFactory *tff;
     GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (feature);
-    gchar *str;
 
-    /* we copy the caps here so we can simplify them before saving. This is a lot
-     * faster when loading them later on */
-    GstCaps *copy = gst_caps_copy (factory->caps);
-
-    tff = g_malloc (sizeof (GstBinaryTypeFindFactory));
+    tff = g_malloc0 (sizeof (GstBinaryTypeFindFactory));
     chk =
         gst_registry_binary_make_data (tff, sizeof (GstBinaryTypeFindFactory));
     tff->nextensions = 0;
@@ -519,14 +547,25 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
       }
     }
     /* save caps */
-    gst_caps_do_simplify (copy);
-    str = gst_caps_to_string (copy);
-    gst_caps_unref (copy);
-    gst_registry_binary_save_string (list, str);
+    if (factory->caps) {
+      GstCaps *copy = gst_caps_copy (factory->caps);
+      gchar *str;
+
+      /* we copy the caps here so we can simplify them
+       * before saving. This is a lot faster when loading
+       * them later on */
+      gst_caps_do_simplify (copy);
+      str = gst_caps_to_string (copy);
+      gst_caps_unref (copy);
+      gst_registry_binary_save_string (list, str);
+    } else {
+      gst_registry_binary_save_const_string (list, "");
+    }
+
   } else if (GST_IS_INDEX_FACTORY (feature)) {
     GstIndexFactory *factory = GST_INDEX_FACTORY (feature);
 
-    pf = g_malloc (sizeof (GstBinaryPluginFeature));
+    pf = g_malloc0 (sizeof (GstBinaryPluginFeature));
     chk = gst_registry_binary_make_data (pf, sizeof (GstBinaryPluginFeature));
     pf->rank = feature->rank;
 
@@ -573,13 +612,13 @@ gst_registry_binary_save_plugin_dep (GList ** list, GstPluginDep * dep)
   ed->stat_hash = dep->stat_hash;
 
   for (s = dep->env_vars; s != NULL && *s != NULL; ++s, ++ed->n_env_vars)
-    gst_registry_binary_save_string (list, *s);
+    gst_registry_binary_save_string (list, g_strdup (*s));
 
   for (s = dep->paths; s != NULL && *s != NULL; ++s, ++ed->n_paths)
-    gst_registry_binary_save_string (list, *s);
+    gst_registry_binary_save_string (list, g_strdup (*s));
 
   for (s = dep->names; s != NULL && *s != NULL; ++s, ++ed->n_names)
-    gst_registry_binary_save_string (list, *s);
+    gst_registry_binary_save_string (list, g_strdup (*s));
 
   *list = g_list_prepend (*list, chk);
 
@@ -602,7 +641,7 @@ gst_registry_binary_save_plugin (GList ** list, GstRegistry * registry,
   GList *plugin_features = NULL;
   GList *walk;
 
-  pe = g_malloc (sizeof (GstBinaryPluginElement));
+  pe = g_malloc0 (sizeof (GstBinaryPluginElement));
   chk = gst_registry_binary_make_data (pe, sizeof (GstBinaryPluginElement));
 
   pe->file_size = plugin->file_size;
@@ -635,6 +674,14 @@ gst_registry_binary_save_plugin (GList ** list, GstRegistry * registry,
 
   gst_plugin_feature_list_free (plugin_features);
 
+  /* pack cache data */
+  if (plugin->priv->cache_data) {
+    gchar *cache_str = gst_structure_to_string (plugin->priv->cache_data);
+    gst_registry_binary_save_string (list, cache_str);
+  } else {
+    gst_registry_binary_save_const_string (list, "");
+  }
+
   /* pack plugin element strings */
   gst_registry_binary_save_const_string (list, plugin->desc.origin);
   gst_registry_binary_save_const_string (list, plugin->desc.package);
@@ -692,10 +739,9 @@ gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
       continue;
 
     if (plugin->flags & GST_PLUGIN_FLAG_CACHED) {
-      int ret;
       struct stat statbuf;
 
-      if ((ret = g_stat (plugin->filename, &statbuf)) < 0 ||
+      if (g_stat (plugin->filename, &statbuf) < 0 ||
           plugin->file_mtime != statbuf.st_mtime ||
           plugin->file_size != statbuf.st_size)
         continue;
@@ -728,10 +774,6 @@ gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
 
     if (!gst_registry_binary_write_chunk (registry, cache, cur->data, cur->size,
             &file_position, cur->align)) {
-      if (!(cur->flags & GST_BINARY_REGISTRY_FLAG_CONST))
-        g_free (cur->data);
-      g_free (cur);
-      walk->data = NULL;
       goto fail_free_list;
     }
     if (!(cur->flags & GST_BINARY_REGISTRY_FLAG_CONST))
@@ -786,12 +828,8 @@ gst_registry_binary_check_magic (gchar ** in, gsize size)
 
   align (*in);
   GST_DEBUG ("Reading/casting for GstBinaryRegistryMagic at address %p", *in);
-  unpack_element (*in, m, GstBinaryRegistryMagic);
+  unpack_element (*in, m, GstBinaryRegistryMagic, (*in + size), fail);
 
-  if (m == NULL || m->magic == NULL || m->version == NULL) {
-    GST_WARNING ("Binary registry magic structure is broken");
-    return -1;
-  }
   if (strncmp (m->magic, GST_MAGIC_BINARY_REGISTRY_STR,
           GST_MAGIC_BINARY_REGISTRY_LEN) != 0) {
     GST_WARNING
@@ -811,6 +849,10 @@ gst_registry_binary_check_magic (gchar ** in, gsize size)
   }
 
   return 0;
+
+fail:
+  GST_WARNING ("Not enough data for binary registry magic structure");
+  return -1;
 }
 
 
@@ -822,27 +864,33 @@ gst_registry_binary_check_magic (gchar ** in, gsize size)
  * Returns: new GstStaticPadTemplate
  */
 static gboolean
-gst_registry_binary_load_pad_template (GstElementFactory * factory, gchar ** in)
+gst_registry_binary_load_pad_template (GstElementFactory * factory, gchar ** in,
+    gchar * end)
 {
   GstBinaryPadTemplate *pt;
-  GstStaticPadTemplate *template;
+  GstStaticPadTemplate *template = NULL;
 
   align (*in);
   GST_DEBUG ("Reading/casting for GstBinaryPadTemplate at address %p", *in);
-  unpack_element (*in, pt, GstBinaryPadTemplate);
+  unpack_element (*in, pt, GstBinaryPadTemplate, end, fail);
 
   template = g_new0 (GstStaticPadTemplate, 1);
   template->presence = pt->presence;
   template->direction = pt->direction;
 
   /* unpack pad template strings */
-  unpack_const_string (*in, template->name_template);
-  unpack_string (*in, template->static_caps.string);
+  unpack_const_string (*in, template->name_template, end, fail);
+  unpack_string (*in, template->static_caps.string, end, fail);
 
   __gst_element_factory_add_static_pad_template (factory, template);
   GST_DEBUG ("Added pad_template %s", template->name_template);
 
   return TRUE;
+
+fail:
+  GST_INFO ("Reading pad template failed");
+  g_free (template);
+  return FALSE;
 }
 
 
@@ -855,88 +903,96 @@ gst_registry_binary_load_pad_template (GstElementFactory * factory, gchar ** in)
  */
 static gboolean
 gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
-    const gchar * plugin_name)
+    gchar * end, const gchar * plugin_name)
 {
   GstBinaryPluginFeature *pf = NULL;
-  GstPluginFeature *feature;
+  GstPluginFeature *feature = NULL;
   gchar *type_name = NULL, *str;
   GType type;
   guint i;
 
   /* unpack plugin feature strings */
-  unpack_string (*in, type_name);
+  unpack_string (*in, type_name, end, fail);
 
-  if (!type_name || !*(type_name)) {
+  if (G_UNLIKELY (!type_name)) {
     GST_ERROR ("No feature type name");
     return FALSE;
   }
 
   GST_DEBUG ("Plugin '%s' feature typename : '%s'", plugin_name, type_name);
 
-  if (!(type = g_type_from_name (type_name))) {
+  if (G_UNLIKELY (!(type = g_type_from_name (type_name)))) {
     GST_ERROR ("Unknown type from typename '%s' for plugin '%s'", type_name,
         plugin_name);
+    g_free (type_name);
     return FALSE;
   }
-  if ((feature = g_object_new (type, NULL)) == NULL) {
+  if (G_UNLIKELY ((feature = g_object_new (type, NULL)) == NULL)) {
     GST_ERROR ("Can't create feature from type");
+    g_free (type_name);
     return FALSE;
   }
 
-  if (!GST_IS_PLUGIN_FEATURE (feature)) {
+  if (G_UNLIKELY (!GST_IS_PLUGIN_FEATURE (feature))) {
     GST_ERROR ("typename : '%s' is not a plugin feature", type_name);
     goto fail;
   }
 
   /* unpack more plugin feature strings */
-  unpack_string (*in, feature->name);
+  unpack_string (*in, feature->name, end, fail);
 
   if (GST_IS_ELEMENT_FACTORY (feature)) {
     GstBinaryElementFactory *ef;
-    GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
+    guint n;
+    GstElementFactory *factory = GST_ELEMENT_FACTORY_CAST (feature);
 
     align (*in);
     GST_LOG ("Reading/casting for GstBinaryElementFactory at address %p", *in);
-    unpack_element (*in, ef, GstBinaryElementFactory);
+    unpack_element (*in, ef, GstBinaryElementFactory, end, fail);
     pf = (GstBinaryPluginFeature *) ef;
 
     /* unpack element factory strings */
-    unpack_string (*in, factory->details.longname);
-    unpack_string (*in, factory->details.klass);
-    unpack_string (*in, factory->details.description);
-    unpack_string (*in, factory->details.author);
+    unpack_string (*in, factory->details.longname, end, fail);
+    unpack_string (*in, factory->details.klass, end, fail);
+    unpack_string (*in, factory->details.description, end, fail);
+    unpack_string (*in, factory->details.author, end, fail);
+    n = ef->npadtemplates;
     GST_DEBUG ("Element factory : '%s' with npadtemplates=%d",
-        factory->details.longname, ef->npadtemplates);
+        factory->details.longname, n);
 
     /* load pad templates */
-    for (i = 0; i < ef->npadtemplates; i++) {
-      if (!gst_registry_binary_load_pad_template (factory, in)) {
+    for (i = 0; i < n; i++) {
+      if (G_UNLIKELY (!gst_registry_binary_load_pad_template (factory, in,
+                  end))) {
         GST_ERROR ("Error while loading binary pad template");
         goto fail;
       }
     }
 
     /* load uritypes */
-    if (ef->nuriprotocols) {
-      GST_DEBUG ("Reading %d UriTypes at address %p", ef->nuriprotocols, *in);
+    if (G_UNLIKELY ((n = ef->nuriprotocols))) {
+      GST_DEBUG ("Reading %d UriTypes at address %p", n, *in);
 
       align (*in);
       factory->uri_type = *((guint *) * in);
       *in += sizeof (factory->uri_type);
-      //unpack_element(*in, &factory->uri_type, factory->uri_type);
+      /*unpack_element(*in, &factory->uri_type, factory->uri_type, end, fail); */
 
-      factory->uri_protocols = g_new0 (gchar *, ef->nuriprotocols + 1);
-      for (i = 0; i < ef->nuriprotocols; i++) {
-        unpack_string (*in, str);
+      factory->uri_protocols = g_new0 (gchar *, n + 1);
+      for (i = 0; i < n; i++) {
+        unpack_string (*in, str, end, fail);
         factory->uri_protocols[i] = str;
       }
     }
+
     /* load interfaces */
-    GST_DEBUG ("Reading %d Interfaces at address %p", ef->ninterfaces, *in);
-    for (i = 0; i < ef->ninterfaces; i++) {
-      unpack_string (*in, str);
-      __gst_element_factory_add_interface (factory, str);
-      g_free (str);
+    if (G_UNLIKELY ((n = ef->ninterfaces))) {
+      GST_DEBUG ("Reading %d Interfaces at address %p", n, *in);
+      for (i = 0; i < n; i++) {
+        unpack_string (*in, str, end, fail);
+        __gst_element_factory_add_interface (factory, str);
+        g_free (str);
+      }
     }
   } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
     GstBinaryTypeFindFactory *tff;
@@ -944,12 +1000,12 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
 
     align (*in);
     GST_DEBUG ("Reading/casting for GstBinaryPluginFeature at address %p", *in);
-    unpack_element (*in, tff, GstBinaryTypeFindFactory);
+    unpack_element (*in, tff, GstBinaryTypeFindFactory, end, fail);
     pf = (GstBinaryPluginFeature *) tff;
 
     /* load caps */
-    unpack_string (*in, str);
-    factory->caps = gst_caps_from_string (str);
+    unpack_string (*in, str, end, fail);
+    factory->caps = (str && *str) ? gst_caps_from_string (str) : NULL;
     g_free (str);
     /* load extensions */
     if (tff->nextensions) {
@@ -957,7 +1013,7 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
           tff->nextensions, *in);
       factory->extensions = g_new0 (gchar *, tff->nextensions + 1);
       for (i = 0; i < tff->nextensions; i++) {
-        unpack_string (*in, str);
+        unpack_string (*in, str, end, fail);
         factory->extensions[i] = str;
       }
     }
@@ -966,10 +1022,10 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
 
     align (*in);
     GST_DEBUG ("Reading/casting for GstBinaryPluginFeature at address %p", *in);
-    unpack_element (*in, pf, GstBinaryPluginFeature);
+    unpack_element (*in, pf, GstBinaryPluginFeature, end, fail);
 
     /* unpack index factory strings */
-    unpack_string (*in, factory->longdesc);
+    unpack_string (*in, factory->longdesc, end, fail);
   } else {
     GST_WARNING ("unhandled factory type : %s", G_OBJECT_TYPE_NAME (feature));
     goto fail;
@@ -988,16 +1044,19 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
 
   /* Errors */
 fail:
+  GST_INFO ("Reading plugin feature failed");
   g_free (type_name);
-  if (GST_IS_OBJECT (feature))
-    gst_object_unref (feature);
-  else
-    g_object_unref (feature);
+  if (feature) {
+    if (GST_IS_OBJECT (feature))
+      gst_object_unref (feature);
+    else
+      g_object_unref (feature);
+  }
   return FALSE;
 }
 
 static gchar **
-gst_registry_binary_load_plugin_dep_strv (gchar ** in, guint n)
+gst_registry_binary_load_plugin_dep_strv (gchar ** in, gchar * end, guint n)
 {
   gchar **arr;
 
@@ -1006,14 +1065,18 @@ gst_registry_binary_load_plugin_dep_strv (gchar ** in, guint n)
 
   arr = g_new0 (gchar *, n + 1);
   while (n > 0) {
-    unpack_string (*in, arr[n - 1]);
+    unpack_string (*in, arr[n - 1], end, fail);
     --n;
   }
   return arr;
+fail:
+  GST_INFO ("Reading plugin dependency strings failed");
+  return NULL;
 }
 
 static gboolean
-gst_registry_binary_load_plugin_dep (GstPlugin * plugin, gchar ** in)
+gst_registry_binary_load_plugin_dep (GstPlugin * plugin, gchar ** in,
+    gchar * end)
 {
   GstPluginDep *dep;
   GstBinaryDep *d;
@@ -1021,7 +1084,7 @@ gst_registry_binary_load_plugin_dep (GstPlugin * plugin, gchar ** in)
 
   align (*in);
   GST_LOG_OBJECT (plugin, "Unpacking GstBinaryDep from %p", *in);
-  unpack_element (*in, d, GstBinaryDep);
+  unpack_element (*in, d, GstBinaryDep, end, fail);
 
   dep = g_new0 (GstPluginDep, 1);
 
@@ -1030,9 +1093,10 @@ gst_registry_binary_load_plugin_dep (GstPlugin * plugin, gchar ** in)
 
   dep->flags = d->flags;
 
-  dep->names = gst_registry_binary_load_plugin_dep_strv (in, d->n_names);
-  dep->paths = gst_registry_binary_load_plugin_dep_strv (in, d->n_paths);
-  dep->env_vars = gst_registry_binary_load_plugin_dep_strv (in, d->n_env_vars);
+  dep->names = gst_registry_binary_load_plugin_dep_strv (in, end, d->n_names);
+  dep->paths = gst_registry_binary_load_plugin_dep_strv (in, end, d->n_paths);
+  dep->env_vars =
+      gst_registry_binary_load_plugin_dep_strv (in, end, d->n_env_vars);
 
   plugin->priv->deps = g_list_append (plugin->priv->deps, dep);
 
@@ -1046,6 +1110,9 @@ gst_registry_binary_load_plugin_dep (GstPlugin * plugin, gchar ** in)
     GST_LOG_OBJECT (plugin, " name: %s", *s);
 
   return TRUE;
+fail:
+  GST_INFO ("Reading plugin dependency failed");
+  return FALSE;
 }
 
 /*
@@ -1056,15 +1123,17 @@ gst_registry_binary_load_plugin_dep (GstPlugin * plugin, gchar ** in)
  * GstBinaryPluginElement structure.
  */
 static gboolean
-gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
+gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in,
+    gchar * end)
 {
   GstBinaryPluginElement *pe;
   GstPlugin *plugin = NULL;
-  guint i;
+  gchar *cache_str = NULL;
+  guint i, n;
 
   align (*in);
   GST_LOG ("Reading/casting for GstBinaryPluginElement at address %p", *in);
-  unpack_element (*in, pe, GstBinaryPluginElement);
+  unpack_element (*in, pe, GstBinaryPluginElement, end, fail);
 
   plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
 
@@ -1074,14 +1143,14 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
   plugin->file_size = pe->file_size;
 
   /* unpack plugin element strings */
-  unpack_const_string (*in, plugin->desc.name);
-  unpack_string (*in, plugin->desc.description);
-  unpack_string (*in, plugin->filename);
-  unpack_const_string (*in, plugin->desc.version);
-  unpack_const_string (*in, plugin->desc.license);
-  unpack_const_string (*in, plugin->desc.source);
-  unpack_const_string (*in, plugin->desc.package);
-  unpack_const_string (*in, plugin->desc.origin);
+  unpack_const_string (*in, plugin->desc.name, end, fail);
+  unpack_string (*in, plugin->desc.description, end, fail);
+  unpack_string (*in, plugin->filename, end, fail);
+  unpack_const_string (*in, plugin->desc.version, end, fail);
+  unpack_const_string (*in, plugin->desc.license, end, fail);
+  unpack_const_string (*in, plugin->desc.source, end, fail);
+  unpack_const_string (*in, plugin->desc.package, end, fail);
+  unpack_const_string (*in, plugin->desc.origin, end, fail);
   GST_LOG ("read strings for name='%s'", plugin->desc.name);
   GST_LOG ("  desc.description='%s'", plugin->desc.description);
   GST_LOG ("  filename='%s'", plugin->filename);
@@ -1091,25 +1160,36 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
   GST_LOG ("  desc.package='%s'", plugin->desc.package);
   GST_LOG ("  desc.origin='%s'", plugin->desc.origin);
 
+  /* unpack cache data */
+  unpack_string (*in, cache_str, end, fail);
+  if (*cache_str) {
+    plugin->priv->cache_data = gst_structure_from_string (cache_str, NULL);
+  }
+  g_free (cache_str);
+
   plugin->basename = g_path_get_basename (plugin->filename);
 
   /* Takes ownership of plugin */
   gst_registry_add_plugin (registry, plugin);
+  n = pe->nfeatures;
   GST_DEBUG ("Added plugin '%s' plugin with %d features from binary registry",
-      plugin->desc.name, pe->nfeatures);
+      plugin->desc.name, n);
 
   /* Load plugin features */
-  for (i = 0; i < pe->nfeatures; i++) {
-    if (!gst_registry_binary_load_feature (registry, in, plugin->desc.name)) {
+  for (i = 0; i < n; i++) {
+    if (G_UNLIKELY (!gst_registry_binary_load_feature (registry, in, end,
+                plugin->desc.name))) {
       GST_ERROR ("Error while loading binary feature");
+      gst_registry_remove_plugin (registry, plugin);
       goto fail;
     }
   }
 
   /* Load external plugin dependencies */
   for (i = 0; i < pe->n_deps; ++i) {
-    if (!gst_registry_binary_load_plugin_dep (plugin, in)) {
+    if (G_UNLIKELY (!gst_registry_binary_load_plugin_dep (plugin, in, end))) {
       GST_ERROR_OBJECT (plugin, "Could not read external plugin dependency");
+      gst_registry_remove_plugin (registry, plugin);
       goto fail;
     }
   }
@@ -1118,6 +1198,7 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
 
   /* Errors */
 fail:
+  GST_INFO ("Reading plugin failed");
   return FALSE;
 }
 
@@ -1156,7 +1237,7 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
 #endif
 
   mapped = g_mapped_file_new (location, FALSE, &err);
-  if (err != NULL) {
+  if (G_UNLIKELY (err != NULL)) {
     GST_INFO ("Unable to mmap file %s : %s", location, err->message);
     g_error_free (err);
     err = NULL;
@@ -1164,27 +1245,32 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
     g_file_get_contents (location, &contents, &size, &err);
     if (err != NULL) {
       GST_INFO ("Unable to read file %s : %s", location, err->message);
+#ifndef GST_DISABLE_GST_DEBUG
       g_timer_destroy (timer);
+#endif
       g_error_free (err);
       return FALSE;
     }
   } else {
-    if ((contents = g_mapped_file_get_contents (mapped)) == NULL) {
+    if (G_UNLIKELY ((contents = g_mapped_file_get_contents (mapped)) == NULL)) {
       GST_ERROR ("Can't load file %s : %s", location, g_strerror (errno));
       goto Error;
     }
     /* check length for header */
     size = g_mapped_file_get_length (mapped);
   }
+
   /* in is a cursor pointer, we initialize it with the begin of registry and is updated on each read */
   in = contents;
   GST_DEBUG ("File data at address %p", in);
-  if (size < sizeof (GstBinaryRegistryMagic)) {
+  if (G_UNLIKELY (size < sizeof (GstBinaryRegistryMagic))) {
     GST_ERROR ("No or broken registry header");
     goto Error;
   }
+
   /* check if header is valid */
-  if ((check_magic_result = gst_registry_binary_check_magic (&in, size)) < 0) {
+  if (G_UNLIKELY ((check_magic_result =
+              gst_registry_binary_check_magic (&in, size)) < 0)) {
 
     if (check_magic_result == -1)
       GST_ERROR
@@ -1194,20 +1280,21 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
   }
 
   /* check if there are plugins in the file */
-
-  if (!(((gsize) in + sizeof (GstBinaryPluginElement)) <
-          (gsize) contents + size)) {
+  if (G_UNLIKELY (!(((gsize) in + sizeof (GstBinaryPluginElement)) <
+              (gsize) contents + size))) {
     GST_INFO ("No binary plugins structure to read");
     /* empty file, this is not an error */
   } else {
+    gchar *end = contents + size;
+    /* read as long as we still have space for a GstBinaryPluginElement */
     for (;
         ((gsize) in + sizeof (GstBinaryPluginElement)) <
         (gsize) contents + size;) {
       GST_DEBUG ("reading binary registry %" G_GSIZE_FORMAT "(%x)/%"
           G_GSIZE_FORMAT, (gsize) in - (gsize) contents,
           (guint) ((gsize) in - (gsize) contents), size);
-      if (!gst_registry_binary_load_plugin (registry, &in)) {
-        GST_ERROR ("Problem while reading binary registry");
+      if (!gst_registry_binary_load_plugin (registry, &in, end)) {
+        GST_ERROR ("Problem while reading binary registry %s", location);
         goto Error;
       }
     }
@@ -1221,7 +1308,7 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
   GST_INFO ("loaded %s in %lf seconds", location, seconds);
 
   res = TRUE;
-  /* TODO: once we re-use the pointers to registry contents return here */
+  /* TODO: once we re-use the pointers to registry contents, return here */
 
 Error:
 #ifndef GST_DISABLE_GST_DEBUG
