@@ -95,6 +95,7 @@ struct _GstBaseSink {
       GstSegment    *clip_segment;
       /* max amount of time a buffer can be late, -1 no limit. */
       gint64	     max_lateness;
+      gboolean       running;
     } ABI;
     gpointer _gst_reserved[GST_PADDING_LARGE - 1];
   } abidata;
@@ -120,6 +121,8 @@ struct _GstBaseSink {
  * @preroll: Called to present the preroll buffer if desired
  * @render: Called when a buffer should be presented or output, at the
  *     correct moment if the #GstBaseSink has been set to sync to the clock.
+ * @render_list: Same as @render but used whith buffer lists instead of
+ *     buffers. Since: 0.10.24
  * @async_play: Subclasses should override this when they need to perform
  *     special processing when changing to the PLAYING state asynchronously.
  *     Called with the OBJECT_LOCK held.
@@ -179,14 +182,18 @@ struct _GstBaseSinkClass {
   /* Clear a previously indicated unlock request not that unlocking is 
    * complete. Sub-classes should clear any command queue or indicator they
    * set during unlock */
-  gboolean      (*unlock_stop)       (GstBaseSink *sink);
+  gboolean      (*unlock_stop)  (GstBaseSink *sink);
+
+  /* Render a BufferList */
+  GstFlowReturn (*render_list)  (GstBaseSink *sink, GstBufferList *buffer_list);
 
   /*< private >*/
-  gpointer       _gst_reserved[GST_PADDING_LARGE-4];
+  gpointer       _gst_reserved[GST_PADDING_LARGE-5];
 };
 
 GType gst_base_sink_get_type(void);
 
+GstFlowReturn	gst_base_sink_do_preroll 	(GstBaseSink *sink, GstMiniObject *obj);
 GstFlowReturn	gst_base_sink_wait_preroll 	(GstBaseSink *sink);
 
 /* synchronizing against the clock */
@@ -217,7 +224,15 @@ gboolean	gst_base_sink_query_latency 	(GstBaseSink *sink, gboolean *live, gboole
 						 GstClockTime *min_latency, GstClockTime *max_latency);
 GstClockTime	gst_base_sink_get_latency 	(GstBaseSink *sink);
 
-GstClockReturn  gst_base_sink_wait_clock        (GstBaseSink * basesink, GstClockTime time,
+/* render delay */
+void		gst_base_sink_set_render_delay  (GstBaseSink *sink, GstClockTime delay);
+GstClockTime    gst_base_sink_get_render_delay 	(GstBaseSink *sink);
+
+/* blocksize */
+void		gst_base_sink_set_blocksize     (GstBaseSink *sink, guint blocksize);
+guint           gst_base_sink_get_blocksize 	(GstBaseSink *sink);
+
+GstClockReturn  gst_base_sink_wait_clock        (GstBaseSink *sink, GstClockTime time,
                                                  GstClockTimeDiff * jitter);
 GstFlowReturn   gst_base_sink_wait_eos          (GstBaseSink *sink, GstClockTime time,
                                                  GstClockTimeDiff *jitter);
