@@ -86,12 +86,14 @@ static gboolean gst_collect_pads_event (GstPad * pad, GstEvent * event);
 static void gst_collect_pads_finalize (GObject * object);
 static void gst_collect_pads_init (GstCollectPads * pads,
     GstCollectPadsClass * g_class);
+static void ref_data (GstCollectData * data);
+static void unref_data (GstCollectData * data);
+static void gst_collect_pads_check_pads_unlocked (GstCollectPads * pads);
 
 static void
 gst_collect_pads_base_init (gpointer g_class)
 {
-  GST_DEBUG_CATEGORY_INIT (collect_pads_debug, "collectpads", 0,
-      "GstCollectPads");
+  /* Do nothing here */
 }
 
 static void
@@ -99,6 +101,9 @@ gst_collect_pads_class_init (GstCollectPadsClass * klass)
 {
   GObjectClass *gobject_class = (GObjectClass *) klass;
 
+  GST_DEBUG_CATEGORY_INIT (collect_pads_debug, "collectpads", 0,
+      "GstCollectPads");
+
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_collect_pads_finalize);
 }
 
@@ -134,25 +139,11 @@ gst_collect_pads_finalize (GObject * object)
   collected = pads->abidata.ABI.pad_list;
   for (; collected; collected = g_slist_next (collected)) {
     GstCollectData *pdata = (GstCollectData *) collected->data;
-    GstCollectDataDestroyNotify destroy_notify;
 
-    /* FIXME: Ugly hack as we can't add more fields to GstCollectData */
-    destroy_notify = (GstCollectDataDestroyNotify)
-        g_object_get_data (G_OBJECT (pdata->pad),
-        "gst-collect-data-destroy-notify");
-
-    if (destroy_notify)
-      destroy_notify (pdata);
-
-    if (pdata->pad) {
-      GST_DEBUG ("finalize pad %s:%s", GST_DEBUG_PAD_NAME (pdata->pad));
-      gst_object_unref (pdata->pad);
-      pdata->pad = NULL;
-    }
-
-    g_free (pdata);
+    unref_data (pdata);
   }
   /* Free pads list */
+  g_slist_foreach (pads->data, (GFunc) unref_data, NULL);
   g_slist_free (pads->data);
   g_slist_free (pads->abidata.ABI.pad_list);
 
@@ -233,8 +224,6 @@ unref_data (GstCollectData * data)
   if (data->buffer) {
     gst_buffer_unref (data->buffer);
   }
-
-
   g_free (data);
 }
 
@@ -327,7 +316,7 @@ gst_collect_pads_add_pad_full (GstCollectPads * pads, GstPad * pad, guint size,
 
   /* FIXME: Ugly hack as we can't add more fields to GstCollectData */
   g_object_set_data (G_OBJECT (pad), "gst-collect-data-destroy-notify",
-      destroy_notify);
+      (void *) destroy_notify);
 
   GST_COLLECT_PADS_PAD_LOCK (pads);
   GST_OBJECT_LOCK (pad);
@@ -400,13 +389,18 @@ gst_collect_pads_remove_pad (GstCollectPads * pads, GstPad * pad)
   gst_pad_set_element_private (pad, NULL);
   GST_OBJECT_UNLOCK (pad);
 
-  /* backward compat, also remove from data if stopped */
+  /* backward compat, also remove from data if stopped, note that this function
+   * can only be called when we are stopped because we don't take the LOCK to
+   * protect the pads->data list. */
   if (!pads->started) {
     GSList *dlist;
 
     dlist = g_slist_find_custom (pads->data, pad, (GCompareFunc) find_pad);
     if (dlist) {
+      GstCollectData *pdata = dlist->data;
+
       pads->data = g_slist_delete_link (pads->data, dlist);
+      unref_data (pdata);
     }
   }
   /* remove from the pad list */
@@ -513,6 +507,36 @@ gst_collect_pads_collect_range (GstCollectPads * pads, guint64 offset,
   return GST_FLOW_NOT_SUPPORTED;
 }
 
+static gboolean
+gst_collect_pads_is_flushing (GstCollectPads * pads)
+{
+  GSList *walk = NULL;
+  gboolean res = TRUE;
+
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+
+  /* Ensure pads->data state */
+  gst_collect_pads_check_pads_unlocked (pads);
+
+  GST_DEBUG ("Getting flushing state (pads:%p, pads->data:%p)",
+      pads, pads->data);
+
+  for (walk = pads->data; walk; walk = g_slist_next (walk)) {
+    GstCollectData *cdata = walk->data;
+
+    GST_DEBUG_OBJECT (cdata->pad, "flushing:%d", cdata->abidata.ABI.flushing);
+
+    if (cdata->abidata.ABI.flushing) {
+      goto done;
+    }
+  }
+
+  res = FALSE;
+done:
+  GST_COLLECT_PADS_PAD_UNLOCK (pads);
+  return res;
+}
+
 /* FIXME, I think this function is used to work around bad behaviour
  * of elements that add pads to themselves without activating them.
  *
@@ -524,6 +548,8 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
 {
   GSList *walk = NULL;
 
+  GST_DEBUG ("Setting flushing (%d)", flushing);
+
   /* Update the pads flushing flag */
   for (walk = pads->data; walk; walk = g_slist_next (walk)) {
     GstCollectData *cdata = walk->data;
@@ -539,6 +565,10 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
       GST_OBJECT_UNLOCK (cdata->pad);
     }
   }
+  /* Setting the pads to flushing means that we changed the values which
+   * are 'protected' by the cookie. We therefore update it to force a 
+   * recalculation of the current pad status. */
+  pads->abidata.ABI.pad_cookie++;
 }
 
 /**
@@ -561,6 +591,8 @@ gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
   g_return_if_fail (GST_IS_COLLECT_PADS (pads));
 
   GST_COLLECT_PADS_PAD_LOCK (pads);
+  /* Ensure pads->data state */
+  gst_collect_pads_check_pads_unlocked (pads);
   gst_collect_pads_set_flushing_unlocked (pads, flushing);
   GST_COLLECT_PADS_PAD_UNLOCK (pads);
 }
@@ -775,19 +807,20 @@ gst_collect_pads_available (GstCollectPads * pads)
 
     /* ignore pad with EOS */
     if (G_UNLIKELY (pdata->abidata.ABI.eos)) {
-      GST_DEBUG ("pad %p is EOS", pdata);
+      GST_DEBUG ("pad %s:%s is EOS", GST_DEBUG_PAD_NAME (pdata->pad));
       continue;
     }
 
     /* an empty buffer without EOS is weird when we get here.. */
     if (G_UNLIKELY ((buffer = pdata->buffer) == NULL)) {
-      GST_WARNING ("pad %p has no buffer", pdata);
+      GST_WARNING ("pad %s:%s has no buffer", GST_DEBUG_PAD_NAME (pdata->pad));
       goto not_filled;
     }
 
     /* this is the size left of the buffer */
     size = GST_BUFFER_SIZE (buffer) - pdata->pos;
-    GST_DEBUG ("pad %p has %d bytes left", pdata, size);
+    GST_DEBUG ("pad %s:%s has %d bytes left",
+        GST_DEBUG_PAD_NAME (pdata->pad), size);
 
     /* need to return the min of all available data */
     if (size < result)
@@ -813,7 +846,7 @@ not_filled:
  * @size: the number of bytes to read
  *
  * Get a pointer in @bytes where @size bytes can be read from the
- * given pad data.
+ * given pad @data.
  *
  * This function should be called with @pads LOCK held, such as
  * in the callback.
@@ -847,6 +880,75 @@ gst_collect_pads_read (GstCollectPads * pads, GstCollectData * data,
   return readsize;
 }
 
+/**
+ * gst_collect_pads_read_buffer:
+ * @pads: the collectspads to query
+ * @data: the data to use
+ * @size: the number of bytes to read
+ *
+ * Get a subbuffer of @size bytes from the given pad @data.
+ *
+ * This function should be called with @pads LOCK held, such as in the callback.
+ *
+ * Since: 0.10.18
+ *
+ * Returns: A sub buffer. The size of the buffer can be less that requested.
+ * A return of NULL signals that the pad is end-of-stream.
+ * Unref the buffer after use.
+ *
+ * MT safe.
+ */
+GstBuffer *
+gst_collect_pads_read_buffer (GstCollectPads * pads, GstCollectData * data,
+    guint size)
+{
+  guint readsize;
+  GstBuffer *buffer;
+
+  g_return_val_if_fail (pads != NULL, NULL);
+  g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), NULL);
+  g_return_val_if_fail (data != NULL, NULL);
+
+  /* no buffer, must be EOS */
+  if ((buffer = data->buffer) == NULL)
+    return NULL;
+
+  readsize = MIN (size, GST_BUFFER_SIZE (buffer) - data->pos);
+
+  return gst_buffer_create_sub (buffer, data->pos, readsize);
+}
+
+/**
+ * gst_collect_pads_take_buffer:
+ * @pads: the collectspads to query
+ * @data: the data to use
+ * @size: the number of bytes to read
+ *
+ * Get a subbuffer of @size bytes from the given pad @data. Flushes the amount
+ * of read bytes.
+ *
+ * This function should be called with @pads LOCK held, such as in the callback.
+ *
+ * Since: 0.10.18
+ *
+ * Returns: A sub buffer. The size of the buffer can be less that requested.
+ * A return of NULL signals that the pad is end-of-stream.
+ * Unref the buffer after use.
+ *
+ * MT safe.
+ */
+GstBuffer *
+gst_collect_pads_take_buffer (GstCollectPads * pads, GstCollectData * data,
+    guint size)
+{
+  GstBuffer *buffer = gst_collect_pads_read_buffer (pads, data, size);
+
+  if (buffer) {
+    gst_collect_pads_flush (pads, data, GST_BUFFER_SIZE (buffer));
+  }
+  return buffer;
+}
+
 /**
  * gst_collect_pads_flush:
  * @pads: the collectspads to query
@@ -858,7 +960,7 @@ gst_collect_pads_read (GstCollectPads * pads, GstCollectData * data,
  * This function should be called with @pads LOCK held, such as
  * in the callback.
  *
- * Returns: The number of bytes flushed This can be less than @size and
+ * Returns: The number of bytes flushed. This can be less than @size and
  * is 0 if the pad was end-of-stream.
  *
  * MT safe.
@@ -883,6 +985,8 @@ gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
 
   data->pos += size;
 
+  GST_LOG_OBJECT (pads, "Flushing %d bytes, requested %u", flushsize, size);
+
   if (data->pos >= GST_BUFFER_SIZE (buffer))
     /* _clear will also reset data->pos to 0 */
     gst_collect_pads_clear (pads, data);
@@ -900,14 +1004,15 @@ gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
  * Must be called with LOCK.
  */
 static void
-gst_collect_pads_check_pads (GstCollectPads * pads)
+gst_collect_pads_check_pads_unlocked (GstCollectPads * pads)
 {
-  /* the master list and cookie are protected with the PAD_LOCK */
-  GST_COLLECT_PADS_PAD_LOCK (pads);
+  GST_DEBUG ("stored cookie : %d, used_cookie:%d",
+      pads->abidata.ABI.pad_cookie, pads->cookie);
   if (G_UNLIKELY (pads->abidata.ABI.pad_cookie != pads->cookie)) {
     GSList *collected;
 
     /* clear list and stats */
+    g_slist_foreach (pads->data, (GFunc) unref_data, NULL);
     g_slist_free (pads->data);
     pads->data = NULL;
     pads->numpads = 0;
@@ -922,17 +1027,29 @@ gst_collect_pads_check_pads (GstCollectPads * pads)
       /* update the stats */
       pads->numpads++;
       data = collected->data;
-      if (data->buffer)
-        pads->queuedpads++;
-      if (data->abidata.ABI.eos)
-        pads->eospads++;
+
+      if (G_LIKELY (!data->abidata.ABI.flushing)) {
+        if (data->buffer)
+          pads->queuedpads++;
+        if (data->abidata.ABI.eos)
+          pads->eospads++;
+      }
 
       /* add to the list of pads to collect */
+      ref_data (data);
       pads->data = g_slist_prepend (pads->data, data);
     }
     /* and update the cookie */
     pads->cookie = pads->abidata.ABI.pad_cookie;
   }
+}
+
+static inline void
+gst_collect_pads_check_pads (GstCollectPads * pads)
+{
+  /* the master list and cookie are protected with the PAD_LOCK */
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+  gst_collect_pads_check_pads_unlocked (pads);
   GST_COLLECT_PADS_PAD_UNLOCK (pads);
 }
 
@@ -1042,10 +1159,16 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
         pads->eospads--;
         data->abidata.ABI.eos = FALSE;
       }
-      GST_OBJECT_UNLOCK (pads);
 
-      /* forward event */
-      goto forward;
+      if (!gst_collect_pads_is_flushing (pads)) {
+        /* forward event if all pads are no longer flushing */
+        GST_DEBUG ("No more pads are flushing, forwarding FLUSH_STOP");
+        GST_OBJECT_UNLOCK (pads);
+        goto forward;
+      }
+      gst_event_unref (event);
+      GST_OBJECT_UNLOCK (pads);
+      goto done;
     }
     case GST_EVENT_EOS:
     {
@@ -1106,6 +1229,8 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
   }
 
 forward:
+  GST_DEBUG_OBJECT (pads, "forward unhandled event: %s",
+      GST_EVENT_TYPE_NAME (event));
   res = gst_pad_event_default (pad, event);
 
 done:
@@ -1132,7 +1257,6 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstCollectData *data;
   GstCollectPads *pads;
-  guint64 size;
   GstFlowReturn ret;
   GstBuffer **buffer_p;
 
@@ -1147,7 +1271,6 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
   GST_OBJECT_UNLOCK (pad);
 
   pads = data->collect;
-  size = GST_BUFFER_SIZE (buffer);
 
   GST_OBJECT_LOCK (pads);
   /* if not started, bail out */
@@ -1178,6 +1301,7 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
 
   /* While we have data queued on this pad try to collect stuff */
   do {
+    GST_DEBUG ("Pad %s:%s checking", GST_DEBUG_PAD_NAME (pad));
     /* Check if our collected condition is matched and call the collected function
      * if it is */
     ret = gst_collect_pads_check_collected (pads);
@@ -1220,6 +1344,7 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
   while (data->buffer != NULL);
 
 unlock_done:
+  GST_DEBUG ("Pad %s:%s done", GST_DEBUG_PAD_NAME (pad));
   GST_OBJECT_UNLOCK (pads);
   unref_data (data);
   gst_buffer_unref (buffer);
