@@ -37,6 +37,18 @@ typedef struct _GstCollectData GstCollectData;
 typedef struct _GstCollectPads GstCollectPads;
 typedef struct _GstCollectPadsClass GstCollectPadsClass;
 
+/**
+ * GstCollectDataDestroyNotify:
+ * @data: the #GstCollectData that will be freed
+ *
+ * A function that will be called when the #GstCollectData will be freed.
+ * It is passed the pointer to the structure and should free any custom
+ * memory and resources allocated for it.
+ *
+ * Since: 0.10.12
+ */
+typedef void (*GstCollectDataDestroyNotify) (GstCollectData *data);
+
 /**
  * GstCollectData:
  * @collect: owner #GstCollectPads
@@ -62,6 +74,7 @@ struct _GstCollectData
       gboolean           flushing;
       gboolean           new_segment;
       gboolean           eos;
+      gint               refcount;
     } ABI;
     /* adding + 0 to mark ABI change to be undone later */
     gpointer _gst_reserved[GST_PADDING + 0];
@@ -70,12 +83,12 @@ struct _GstCollectData
 
 /**
  * GstCollectPadsFunction:
- * @pads: the #GstCollectPads that trigered the callback
+ * @pads: the #GstCollectPads that triggered the callback
  * @user_data: user data passed to gst_collect_pads_set_function()
  *
  * A function that will be called when all pads have received data.
  *
- * Returns: GST_FLOW_OK for success
+ * Returns: #GST_FLOW_OK for success
  */
 typedef GstFlowReturn (*GstCollectPadsFunction) (GstCollectPads *pads, gpointer user_data);
 
@@ -90,10 +103,9 @@ typedef GstFlowReturn (*GstCollectPadsFunction) (GstCollectPads *pads, gpointer
 
 /**
  * GstCollectPads:
- * @object: the #GstObject parent structure.
  * @data: #GList of #GstCollectData managed by this #GstCollectPads.
  *
- * Collectpads object. 
+ * Collectpads object.
  */
 struct _GstCollectPads {
   GstObject      object;
@@ -148,6 +160,7 @@ void		gst_collect_pads_set_function 	(GstCollectPads *pads, GstCollectPadsFuncti
 
 /* pad management */
 GstCollectData*	gst_collect_pads_add_pad	(GstCollectPads *pads, GstPad *pad, guint size);
+GstCollectData* gst_collect_pads_add_pad_full   (GstCollectPads *pads, GstPad *pad, guint size, GstCollectDataDestroyNotify destroy_notify);
 gboolean	gst_collect_pads_remove_pad	(GstCollectPads *pads, GstPad *pad);
 gboolean	gst_collect_pads_is_active 	(GstCollectPads *pads, GstPad *pad);
 
@@ -157,6 +170,7 @@ GstFlowReturn	gst_collect_pads_collect_range 	(GstCollectPads *pads, guint64 off
 
 void		gst_collect_pads_start 		(GstCollectPads *pads);
 void		gst_collect_pads_stop 		(GstCollectPads *pads);
+void		gst_collect_pads_set_flushing	(GstCollectPads *pads, gboolean flushing);
 
 /* get collected buffers */
 GstBuffer*	gst_collect_pads_peek 		(GstCollectPads *pads, GstCollectData *data);
@@ -166,6 +180,10 @@ GstBuffer*	gst_collect_pads_pop		(GstCollectPads *pads, GstCollectData *data);
 guint 		gst_collect_pads_available 	(GstCollectPads *pads);
 guint		gst_collect_pads_read		(GstCollectPads *pads, GstCollectData *data,
 						 guint8 **bytes, guint size);
+GstBuffer *     gst_collect_pads_read_buffer    (GstCollectPads * pads, GstCollectData * data,
+						 guint size);
+GstBuffer *     gst_collect_pads_take_buffer    (GstCollectPads * pads, GstCollectData * data,
+						 guint size);
 guint 		gst_collect_pads_flush 		(GstCollectPads *pads, GstCollectData *data,
 						 guint size);
 
