@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) 2005 Jan Schmidt <thaytan@mad.scientist.com>
+ *               2009 Wim Taymans <wim.taymans@gmail.com>
  *
  * gstsegment.c: Unit test for segments
  *
@@ -593,6 +594,7 @@ GST_START_TEST (segment_newsegment_open)
   fail_unless (segment.stop == -1);
   fail_unless (segment.time == 0);
   fail_unless (segment.accum == 0);
+  fail_unless (segment.last_stop == 0);
 
   /* update, nothing changes */
   gst_segment_set_newsegment (&segment, TRUE, 1.0, GST_FORMAT_BYTES, 0, -1, 0);
@@ -601,6 +603,7 @@ GST_START_TEST (segment_newsegment_open)
   fail_unless (segment.stop == -1);
   fail_unless (segment.time == 0);
   fail_unless (segment.accum == 0);
+  fail_unless (segment.last_stop == 0);
 
   /* update */
   gst_segment_set_newsegment (&segment, TRUE, 1.0,
@@ -610,6 +613,7 @@ GST_START_TEST (segment_newsegment_open)
   fail_unless (segment.stop == -1);
   fail_unless (segment.time == 100);
   fail_unless (segment.accum == 100);
+  fail_unless (segment.last_stop == 100);
 
   /* last_stop 0, accum does not change */
   gst_segment_set_newsegment (&segment, FALSE, 1.0, GST_FORMAT_BYTES, 0, -1, 0);
@@ -621,6 +625,8 @@ GST_START_TEST (segment_newsegment_open)
 
   gst_segment_set_last_stop (&segment, GST_FORMAT_BYTES, 200);
 
+  fail_unless (segment.last_stop == 200);
+
   /* last_stop 200, accum changes */
   gst_segment_set_newsegment (&segment, FALSE, 1.0, GST_FORMAT_BYTES, 0, -1, 0);
 
@@ -628,7 +634,7 @@ GST_START_TEST (segment_newsegment_open)
   fail_unless (segment.stop == -1);
   fail_unless (segment.time == 0);
   fail_unless (segment.accum == 300);
-
+  fail_unless (segment.last_stop == 0);
 }
 
 GST_END_TEST;
@@ -654,13 +660,29 @@ GST_START_TEST (segment_newsegment_closed)
   fail_unless (segment.last_stop == 0);
   fail_unless (segment.duration == -1);
 
-  /* do an update */
-  gst_segment_set_newsegment (&segment, TRUE, 1.0, GST_FORMAT_BYTES, 0, 300, 0);
+  /* assume we advanced to position 40 */
+  gst_segment_set_last_stop (&segment, GST_FORMAT_BYTES, 40);
+  fail_unless (segment.last_stop == 40);
 
-  fail_unless (segment.start == 0);
+  /* do an update to the start, last_stop is unchanged because it's bigger */
+  gst_segment_set_newsegment (&segment, TRUE, 1.0, GST_FORMAT_BYTES, 20, 200,
+      20);
+
+  fail_unless (segment.start == 20);
+  fail_unless (segment.stop == 200);
+  fail_unless (segment.time == 20);
+  fail_unless (segment.accum == 20);
+  fail_unless (segment.last_stop == 40);
+
+  /* do an update past our last_stop, it should be updated now */
+  gst_segment_set_newsegment (&segment, TRUE, 1.0, GST_FORMAT_BYTES, 50, 300,
+      50);
+
+  fail_unless (segment.start == 50);
   fail_unless (segment.stop == 300);
-  fail_unless (segment.time == 0);
-  fail_unless (segment.accum == 0);
+  fail_unless (segment.time == 50);
+  fail_unless (segment.accum == 50);
+  fail_unless (segment.last_stop == 50);
 
   /* and a new accumulated one */
   gst_segment_set_newsegment (&segment, FALSE, 1.0,
@@ -1222,7 +1244,7 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate_rate)
   fail_unless (segment.time == 200);
   /* accumulated 100 of previous segment to make 200 */
   fail_unless (segment.accum == 200);
-  fail_unless (segment.last_stop == 0);
+  fail_unless (segment.last_stop == 200);
   fail_unless (segment.duration == -1);
 
   /* invalid time gives invalid result */
@@ -1262,7 +1284,7 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate_rate)
   fail_unless (segment.stop == 200);
   fail_unless (segment.time == 0);
   fail_unless (segment.accum == 400);
-  fail_unless (segment.last_stop == 0);
+  fail_unless (segment.last_stop == 200);
   fail_unless (segment.duration == -1);
 
   /* invalid time gives invalid result */
@@ -1320,17 +1342,25 @@ GST_START_TEST (segment_newsegment_runningtime)
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 0);
   fail_unless (result == 0);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 0);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 100);
   fail_unless (result == 100);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 100);
 
   /* at edge is exactly the segment duration */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 200);
   fail_unless (result == 200);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 200);
 
   /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, 300);
+  fail_unless (result == -1);
 
   /***********************************************************
    * time shifted by 500, check if accumulation worked.
@@ -1350,13 +1380,19 @@ GST_START_TEST (segment_newsegment_runningtime)
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 0);
   fail_unless (result == 200);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 0);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 100);
   fail_unless (result == 250);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 100);
 
   /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 500);
   fail_unless (result == -1);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, 310);
+  fail_unless (result == -1);
 
   /********************************************
    * time offset by 500
@@ -1376,19 +1412,29 @@ GST_START_TEST (segment_newsegment_runningtime)
   /* before segment is invalid */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 400);
   fail_unless (result == -1);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, 200);
+  fail_unless (result == -1);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 500);
   fail_unless (result == 300);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 500);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 600);
   fail_unless (result == 400);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 600);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 700);
   fail_unless (result == 500);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 700);
 
   /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 800);
   fail_unless (result == -1);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, 600);
+  fail_unless (result == -1);
 
   /**********************************************************
    * time offset by 500, shifted by 200
@@ -1408,19 +1454,29 @@ GST_START_TEST (segment_newsegment_runningtime)
   /* before segment is invalid */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 400);
   fail_unless (result == -1);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, 400);
+  fail_unless (result == -1);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 500);
   fail_unless (result == 700);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 500);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 600);
   fail_unless (result == 600);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 600);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 700);
   fail_unless (result == 500);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 700);
 
   /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 800);
   fail_unless (result == -1);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, 800);
+  fail_unless (result == -1);
 
   /**********************************************************
    * time offset by 500, shifted by 200
@@ -1440,20 +1496,30 @@ GST_START_TEST (segment_newsegment_runningtime)
   /* before segment is invalid */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 400);
   fail_unless (result == -1);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, 600);
+  fail_unless (result == -1);
 
   /* total scaled segment time is 100, accum is 700, so we get 800 */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 500);
   fail_unless (result == 800);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 500);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 600);
   fail_unless (result == 750);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 600);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 700);
   fail_unless (result == 700);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 700);
 
   /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 800);
   fail_unless (result == -1);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, 900);
+  fail_unless (result == -1);
 
   /* see if negative rate closed segment correctly */
   gst_segment_set_newsegment_full (&segment, FALSE, -2.0, -1.0,
@@ -1461,6 +1527,8 @@ GST_START_TEST (segment_newsegment_runningtime)
 
   /* previous segment lasted 100, and was at 700 so we should get 800 */
   fail_unless (segment.accum == 800);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, 800);
+  fail_unless (result == 700);
 }
 
 GST_END_TEST;
@@ -1487,7 +1555,7 @@ GST_START_TEST (segment_newsegment_accum)
   fail_unless (segment.stop == 200);
   fail_unless (segment.time == 0);
   fail_unless (segment.accum == 0);
-  fail_unless (segment.last_stop == 0);
+  fail_unless (segment.last_stop == 200);
   fail_unless (segment.duration == -1);
 
   /* invalid time gives invalid result */
@@ -1496,9 +1564,13 @@ GST_START_TEST (segment_newsegment_accum)
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 200);
   fail_unless (result == 0);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 200);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 50);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 150);
 
   /* update segment, this accumulates 50 from the previous segment. */
   gst_segment_set_newsegment_full (&segment, TRUE, -2.0, 1.0,
@@ -1512,15 +1584,19 @@ GST_START_TEST (segment_newsegment_accum)
   fail_unless (segment.stop == 150);
   fail_unless (segment.time == 0);
   fail_unless (segment.accum == 50);
-  fail_unless (segment.last_stop == 0);
+  fail_unless (segment.last_stop == 150);
   fail_unless (segment.duration == -1);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 50);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 150);
 
   /* 50 accumulated + 50 / 2 */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 100);
   fail_unless (result == 75);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 100);
 
   /* update segment, this does not accumulate anything. */
   gst_segment_set_newsegment_full (&segment, TRUE, 1.0, 1.0,
@@ -1534,14 +1610,18 @@ GST_START_TEST (segment_newsegment_accum)
   fail_unless (segment.stop == 200);
   fail_unless (segment.time == 100);
   fail_unless (segment.accum == 50);
-  fail_unless (segment.last_stop == 100);
+  fail_unless (segment.last_stop == 150);
   fail_unless (segment.duration == -1);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 100);
   fail_unless (result == 50);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 100);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 100);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 150);
 }
 
 GST_END_TEST;
@@ -1568,18 +1648,24 @@ GST_START_TEST (segment_newsegment_accum2)
   fail_unless (segment.stop == 200);
   fail_unless (segment.time == 0);
   fail_unless (segment.accum == 0);
-  fail_unless (segment.last_stop == 0);
+  fail_unless (segment.last_stop == 200);
   fail_unless (segment.duration == -1);
 
   /* invalid time gives invalid result */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, -1);
   fail_unless (result == -1);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == -1);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 200);
   fail_unless (result == 0);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 200);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 50);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 150);
 
   /* close segment, this accumulates nothing. */
   gst_segment_set_newsegment_full (&segment, TRUE, -1.0, 1.0,
@@ -1593,7 +1679,7 @@ GST_START_TEST (segment_newsegment_accum2)
   fail_unless (segment.stop == 200);
   fail_unless (segment.time == 0);
   fail_unless (segment.accum == 0);
-  fail_unless (segment.last_stop == 150);
+  fail_unless (segment.last_stop == 200);
   fail_unless (segment.duration == -1);
 
   /* new segment, this accumulates 50. */
@@ -1617,14 +1703,41 @@ GST_START_TEST (segment_newsegment_accum2)
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 50);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 150);
 
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 200);
   fail_unless (result == 100);
+  result = gst_segment_to_position (&segment, GST_FORMAT_TIME, result);
+  fail_unless (result == 200);
 }
 
 GST_END_TEST;
 
-Suite *
+GST_START_TEST (segment_copy)
+{
+  GstSegment *copy;
+  GstSegment segment = { 0.0, };
+
+  /* this is a boxed type copy function, we support copying NULL */
+  fail_unless (gst_segment_copy (NULL) == NULL);
+
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+
+  gst_segment_set_newsegment_full (&segment, FALSE, -1.0, 1.0,
+      GST_FORMAT_TIME, 0, 200, 0);
+
+  copy = gst_segment_copy (&segment);
+  fail_unless (copy != NULL);
+  /* we inited the struct on the stack to zeroes, so direct comparison should
+   * be ok here despite the padding field and regardless of implementation */
+  fail_unless (memcmp (copy, &segment, sizeof (GstSegment)) == 0);
+  gst_segment_free (copy);
+}
+
+GST_END_TEST;
+
+static Suite *
 gst_segment_suite (void)
 {
   Suite *s = suite_create ("GstSegment");
@@ -1646,6 +1759,7 @@ gst_segment_suite (void)
   tcase_add_test (tc_chain, segment_newsegment_runningtime);
   tcase_add_test (tc_chain, segment_newsegment_accum);
   tcase_add_test (tc_chain, segment_newsegment_accum2);
+  tcase_add_test (tc_chain, segment_copy);
 
   return s;
 }
